{-# LANGUAGE GeneralizedNewtypeDeriving #-}

module Deus.Random where

import Control.Monad.State.Strict

import System.Random

import Control.Applicative

-- simple random monad

instance Applicative (State s) where 
  pure = return
  (<*>) = ap
  
newtype RandomM g a = RandomM (State g a) deriving (Functor,Applicative,Monad)

runRandomM :: RandomGen g => RandomM g a -> g -> (a,g)
runRandomM (RandomM a) = runState a

evalRandomM :: RandomGen g => g -> RandomM g a -> a
evalRandomM gen rm = let (a,g) = runRandomM rm gen  in g `seq` a

runRandomMIO a = getStdRandom (runRandomM a)
                    
randomM :: (RandomGen g, Random a) => RandomM g a
randomM = RandomM (State (random))

randomMR rng = RandomM (State (randomR rng))

randomListM :: (RandomGen g) => [a] -> RandomM g a
randomListM l = do 
  n <- randomMR (0,length l-1)
  return (l !! n)
